@@ -19,22 +19,24 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef __GNOPIINIT__MODULE
-#define __GNOPIINIT__MODULE
+#ifndef __GNOPIINIT_MODULE__
+#define __GNOPIINIT_MODULE__
 
 #include <glib.h>
 #include <stdio.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 
-gboolean	brl_set_int(gint val,const gchar *key);
+gboolean brl_set_int                 (gint        val,
+				     const gchar  *key);
 
-gboolean	brl_set_string		(const gchar *val,const gchar *key);
-gboolean 	brl_gconf_client_init	();
+gboolean brl_set_string		     (const gchar *val,
+                                     const gchar  *key);
+gboolean brl_gconf_client_init	     ();
 
-gint 
-brl_get_int_with_default(const gchar *key,gint default_value);
-gchar* 
-brl_get_string_with_default(const gchar *key,gchar *default_value);
+gint     brl_get_int_with_default    (const gchar *key,
+                                     gint         default_value);
+gchar*   brl_get_string_with_default (const gchar *key,
+				     gchar        *default_value);
 
 #endif
