@@ -107,7 +107,7 @@ GtkWidget		*w_braille_settings = NULL;
 **/
 GtkWidget		*w_braille_device;
 static GtkWidget 	*cb_braille_device;
-static GtkWidget 	*sp_port;
+static GtkWidget 	*cb_port;
 static GtkWidget	*lb_port;
 
 /**
@@ -310,6 +310,7 @@ brlui_braille_device_changeing (Braille *braille_setting)
 {
     gchar *tmp = NULL;
     gchar *id = NULL;
+    GtkTreeIter iter;
     
     sru_return_if_fail (braille_setting);    
     
@@ -318,8 +319,20 @@ brlui_braille_device_changeing (Braille *braille_setting)
     id  = (gchar*)brlconf_device_id_return (tmp);	 
     if (id)
 	brlconf_device_set (id);
-    braille_setting->port_no = 
-	gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (sp_port));
+
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (cb_port), &iter))
+    {
+	gchar *str, *error;
+	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (cb_port)),
+			    &iter, 0, &str, -1);
+	braille_setting->port_no = strtol (str, &error, 10);
+	if ((error && *error) || braille_setting->port_no < MIN_SERIAL_PORT_NO ||
+		braille_setting->port_no > MAX_SERIAL_PORT_NO)
+	    braille_setting->port_no = -1;
+	g_free (str);
+    }
+    else
+        braille_setting->port_no = -1;
     brlconf_port_no_set (braille_setting->port_no);
 }
 
@@ -340,7 +353,7 @@ brlui_braille_device_combo_changed (GtkWidget *entry,
 	sensitivity = TRUE;
 	
     gtk_widget_set_sensitive (GTK_WIDGET (lb_port), sensitivity);
-    gtk_widget_set_sensitive (GTK_WIDGET (sp_port), sensitivity);
+    gtk_widget_set_sensitive (GTK_WIDGET (cb_port), sensitivity);
     
 #if 0    /* remove when it need to select the tty number */
     if (!strcmp (BRLTTY, gtk_entry_get_text (GTK_ENTRY (entry))))
@@ -399,7 +412,7 @@ brlui_set_handlers_braille_device (GladeXML *xml,
 {
     w_braille_device  = glade_xml_get_widget (xml, "w_braille_device");
     cb_braille_device = glade_xml_get_widget (xml, "cb_braille_device");
-    sp_port 	      = glade_xml_get_widget (xml, "sp_port");
+    cb_port 	      = glade_xml_get_widget (xml, "cb_port");
     lb_port 	      = glade_xml_get_widget (xml, "lb_port");
 
     g_signal_connect (w_braille_device, "response",
@@ -421,6 +434,7 @@ brlui_load_braille_device (Braille *braille_setting)
     if (!w_braille_device)
     {
 	GladeXML *xml;
+	gint i;
 	xml = gn_load_interface ("Braille_Settings/braille_device.glade2", NULL);
 	sru_return_val_if_fail (xml, FALSE);
     	brlui_set_handlers_braille_device (xml, braille_setting);
@@ -429,6 +443,14 @@ brlui_load_braille_device (Braille *braille_setting)
 				      GTK_WINDOW (w_braille_settings));
 	gtk_window_set_destroy_with_parent (GTK_WINDOW (w_braille_device), 
 						TRUE);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (cb_port), _("<none>"));
+	for (i = 0; i <= MAX_SERIAL_PORT_NO; i++)
+	{
+	    gchar *port;
+	    port = g_strdup_printf ("%d", i);
+	    gtk_combo_box_append_text (GTK_COMBO_BOX (cb_port), port);
+	    g_free (port);
+	}
     }
     else
 	gtk_widget_show (w_braille_device);
@@ -451,6 +473,12 @@ brlui_braille_device_set_to_combo (GList *brldev_list)
 gboolean 
 brlui_braille_device_value_add_to_widgets (Braille *braille_setting)
 {    
+    GtkTreeModel* model;
+    GtkTreeIter iter, iter_found;
+    gboolean found = FALSE, hasnone = FALSE;
+    gint port;
+    gchar *str, *error;
+
     sru_return_val_if_fail (braille_setting, FALSE);    
     
     if (!w_braille_device)
@@ -462,8 +490,35 @@ brlui_braille_device_value_add_to_widgets (Braille *braille_setting)
 
     gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (cb_braille_device)->entry), 
 			brlconf_return_active_device ());
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON (sp_port), 
-			       (gdouble)braille_setting->port_no);
+
+    model = gtk_combo_box_get_model (GTK_COMBO_BOX (cb_port));
+    if (!model)
+	return FALSE;
+    if (!gtk_tree_model_get_iter_first (model, &iter))
+	return FALSE;
+
+    while (!found)
+    {
+	gtk_tree_model_get (model, &iter, 0, &str, -1);
+	if (strcmp (str, _("<none>")) == 0 && !found)
+	{
+	    iter_found = iter;
+	    hasnone= TRUE;
+	}
+	port = strtol (str, &error, 10);
+	if ((!error || !*error) && port == braille_setting->port_no)
+	{
+	    sru_assert (found == FALSE);
+	    iter_found = iter;
+	    found = TRUE;
+	}
+	g_free (str);
+	if (!gtk_tree_model_iter_next (model, &iter))
+	    break;
+    };
+    if (found || hasnone)
+	gtk_combo_box_set_active_iter (GTK_COMBO_BOX (cb_port), &iter_found);
+
     return TRUE;
 }
 
@@ -529,7 +584,7 @@ brlui_translation_table_changeing (Braille *braille_setting)
 
     for (l_iter = 0 ; trans_table_list [ l_iter ].state ; l_iter++)
     {
-	if (!strcmp (key, trans_table_list [ l_iter ].state))
+	if (!strcmp (key, _(trans_table_list [ l_iter ].state)))
 	{
 	    g_free (braille_setting->translation_table);
 	    braille_setting->translation_table = g_strdup (trans_table_list [ l_iter ].file_name);
