@@ -1519,6 +1519,24 @@ src_speech_set_default_values (SRCSpeech *src_speech)
     src_speech->count_mode         = SRC_SPEECH_COUNT_NONE;
 }
 
+static gboolean
+src_timeout_start_speech (gpointer data)
+{
+    gboolean use_speech = TRUE;
+    SET_SRCORE_CONFIG_DATA (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, &use_speech);
+    return FALSE;    
+}
+
+#define SRC_SPEECH_RESTART_TIMEOUT 1000
+static void
+src_restart_speech ()
+{
+    g_printerr ("\nRestarting speech.\n");
+    gboolean use_speech = FALSE;
+    SET_SRCORE_CONFIG_DATA (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, &use_speech);
+    g_timeout_add (SRC_SPEECH_RESTART_TIMEOUT, src_timeout_start_speech, NULL);
+}
+
 gboolean
 src_speech_init ()
 {
@@ -1537,7 +1555,7 @@ src_speech_init ()
     src_speech_set_default_values (src_speech);
     
     srcs_in_init ();
-    src_speech_initialized = srs_init (srcs_in_parse_input);
+    src_speech_initialized = srs_init (srcs_in_parse_input, src_restart_speech);
 #ifdef SRS_NO_MARKERS_SUPPORTED
     src_speech_callback = TRUE;
     src_speech_unspoken = NULL;
