@@ -181,6 +181,7 @@ typedef struct _SRLLastInfo
     SRLong	text_selections_count;
     SRLong	text_crt_selection_length;
     gdouble	value_crt_value;
+    gboolean	checked;
 }SRLLastInfo;
 
 typedef gboolean (*SRLNotify) (SRLEvent *event, SREventType type);
@@ -649,7 +650,6 @@ srle_set_acc (SRLEvent *event)
 			Accessible_unref (child);
 		}
 	    }
-	    
 	    if (!is_focused_child && srle_acc_has_role (event, SPI_ROLE_TABLE))
 	    {
 		for (i = 0; i < cnt; i++)
@@ -985,6 +985,8 @@ srl_notify_clients_obj (SRLEvent *event,
 		    sre_release_reference (pev);
 		}
 	    }
+	    else if (srle_has_type (event, SRL_EVENT_FOCUS))
+		srl_set_last_table (NULL);
 	    if (pacc)
 		Accessible_unref (pacc);
 	}
@@ -1342,7 +1344,7 @@ srl_get_context (Accessible *acc)
     context = NULL;
     Accessible_ref (acc);
     acc_role = Accessible_getRole (acc);
-    if (acc_role == SPI_ROLE_TABLE && acc != srl_last_table)
+    if (acc_role == SPI_ROLE_TABLE)
     {
 	context = acc;
 	Accessible_ref (context);
@@ -1570,6 +1572,17 @@ srl_set_value_info (AccessibleValue *value,
     return TRUE;
 }
 
+static gboolean
+srl_set_checked_info (Accessible *acc,
+		      SRLLastInfo *info)
+{
+    srl_assert (acc && info);
+
+    info->checked = srl_acc_has_state (acc, SPI_STATE_CHECKED);
+
+    return TRUE;
+}
+
 static gboolean
 srl_set_info (Accessible *acc,
 	      SRLLastInfo *info)
@@ -1593,6 +1606,7 @@ srl_set_info (Accessible *acc,
         AccessibleText_unref (text);
     }
 
+    srl_set_checked_info (acc, info);
     return TRUE;
 }
 
@@ -1749,14 +1763,13 @@ srl_selection_event_is_reported (SRLEvent *event)
 }
 
 
+static SRLLastInfo last_info_focus, last_info_watched;
 
 static gboolean
 srl_event_is_reported (SRLEvent *event)
 {
     gboolean rv = FALSE;
-    static SRLLastInfo last_info_focus, last_info_watched;
     SRLLastInfo *last_info = NULL;
-
     srl_assert (event);
     last_info = srle_is_for_focused_acc (event) ? &last_info_focus : &last_info_watched;
 
@@ -1791,7 +1804,8 @@ srl_event_is_reported (SRLEvent *event)
 	rv = TRUE;
     else if (srle_has_type (event, SRL_EVENT_FOCUS))
 	rv = srle_acc_has_state (event, SPI_STATE_FOCUSED) ||
-	     srle_acc_has_state (event, SPI_STATE_TRANSIENT);
+	     srle_acc_has_state (event, SPI_STATE_TRANSIENT) ||
+	     srle_acc_has_state (event, SPI_STATE_ACTIVE);
     else if (srle_has_type (event, SRL_EVENT_FOCUS1))
 	rv = TRUE;
     else if (srle_has_type (event, SRL_EVENT_FOCUS2))
@@ -1802,12 +1816,16 @@ srl_event_is_reported (SRLEvent *event)
 	if (srle_has_type (event, SRL_EVENT_STATE_EXPANDED)) 
 	    rv = srle_acc_has_role (event, SPI_ROLE_TABLE_CELL);
 	else if (srle_has_type (event, SRL_EVENT_STATE_CHECKED))
-	    rv = srle_acc_has_role (event, SPI_ROLE_CHECK_BOX)		||
-		    srle_acc_has_role (event, SPI_ROLE_RADIO_BUTTON)	||
+	{
+	    if (srle_acc_has_role (event, SPI_ROLE_RADIO_BUTTON))
+		rv = !last_info->checked && srle_acc_has_state (event, SPI_STATE_CHECKED);
+	    else
+		rv = srle_acc_has_role (event, SPI_ROLE_CHECK_BOX)	||
 		    srle_acc_has_role (event, SPI_ROLE_TABLE_CELL)	||
 		    srle_acc_has_role (event, SPI_ROLE_TOGGLE_BUTTON)	||
 		    srle_acc_has_role (event, SPI_ROLE_RADIO_MENU_ITEM)	||
 		    srle_acc_has_role (event, SPI_ROLE_CHECK_MENU_ITEM);
+	}
 /*FIXME: next code is needed? */
 /*	else if (srle_has_type (event, SRL_EVENT_STATE_SELECTED))
 	    rv = srle_acc_has_role (event, SPI_ROLE_PAGE_TAB); */
@@ -2256,3 +2274,115 @@ srl_is_object_focused (SRObject *obj)
 
     return sro_get_acc (obj) == srl_last_focus || sro_get_acc (obj) == srl_last_focus2;
 }
+
+static Accessible*
+srl_get_focused_from_acc (Accessible *acc)
+{
+    Accessible *focused = NULL;
+    gint cnt, i;
+
+    srl_assert (acc);
+    
+    cnt = Accessible_getChildCount (acc);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child;
+	child = Accessible_getChildAtIndex (acc, i);
+	if (!child)
+	    continue;
+	if (srl_acc_has_state (child, SPI_STATE_FOCUSED))
+	{
+	    focused = child;
+	    Accessible_ref (focused);
+	}
+	else
+	    focused = srl_get_focused_from_acc (child);
+	
+	Accessible_unref (child);
+	if (focused)
+	    break;
+    }
+
+    return focused;
+}
+
+
+static Accessible*
+srl_get_focused_from_active_toplevel (Accessible *window)
+{
+    return srl_get_focused_from_acc (window);
+}
+
+static Accessible*
+srl_get_active_toplevel_from_app (Accessible *app)
+{
+    Accessible *active = NULL;
+    gint cnt, i;
+
+    srl_assert (app);
+    
+    cnt = Accessible_getChildCount (app);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child;
+	child = Accessible_getChildAtIndex (app, i);
+	if (!child)
+	    continue;
+	if (srl_acc_has_state (child, SPI_STATE_ACTIVE))
+	{
+	    active = child;
+	    break;
+	}
+	Accessible_unref (child);
+    }
+
+    return active;
+}
+
+static Accessible*
+srl_get_focused_from_desktop ()
+{
+    Accessible *desktop, *focused = NULL;
+    gint cnt, i;
+
+    desktop = SPI_getDesktop (0);
+    if (!desktop)
+	return NULL;
+    cnt = Accessible_getChildCount (desktop);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child = NULL, *active = NULL;
+	child = Accessible_getChildAtIndex (desktop, i);
+	if (child)
+	    active = srl_get_active_toplevel_from_app (child);
+	if (active)
+	    focused = srl_get_focused_from_active_toplevel (active);
+	if (child)
+	    Accessible_unref (child);
+	if (active)
+	    Accessible_unref (active);
+	if (focused)
+	    break;
+    }
+    Accessible_unref (desktop);
+
+    return focused;
+}
+
+SRObject*
+srl_get_focus_object ()
+{
+    SRObject *sro = NULL;
+    Accessible *focused;
+
+    focused = srl_get_focused_from_desktop ();
+    if (focused)
+    {
+	srl_last_focus = focused;
+	srl_set_info (focused, &last_info_focus);
+	if (!sro_get_from_acc (focused, SRL_EVENT_NAME_FOCUS, &sro))
+	    sro = NULL;
+    }
+
+    return sro;
+}
